class customized_callbacks = object(self)
  inherit Blabla_glade_callbacks.default_callbacks

  method on_quit_item_activate () = GtkMain.Main.quit ()


  method on_new_game_item_activate () =
    begin
    match !Global.dictionary with
     | Some dict ->
	 begin
	 (* On arrte le timer dans le cas o le joueur dmarre une *)
	 (* nouvelle partie alors qu'il y en a dj une en cours.   *)
	 Ihm_misc.stop_timer () ;
	 Global.score := 0 ;
	 Global.nb_found := 0 ;
	 Global.letters :=
	   Game.choose_letters
	     Global.current_prefs.Global.nb_letters
	     Global.current_prefs.Global.ration_cons_voy ;
	 Global.possibilities :=
	   Game.combine dict !Global.initial_base !Global.letters ;
	 Global.nb_possibilities := List.length !Global.possibilities ;
	 (* On r-initialise la liste des propositions et solutions. *)
	 (match !Global.solutions_clist with
	  | None -> assert false
	  | Some clist -> clist#clear ()) ;
	 (* On initialise la progressbar de pourcentage de mots trouvs. *)
	 self#top_play_window#player_progressbar#configure
	   ~current: 0.0 ~min: 0.0
	   ~max: (float_of_int !Global.nb_possibilities) ;
	 self#top_play_window#score_label#set_text "0" ;
	 self#top_play_window#letters_label#set_text
	   (Game.letters_to_string !Global.letters) ;
	 (* On rend actives les widgets de jeu. *)
	 self#top_play_window#proposition_entry#misc#set_sensitive true ;
	 self#top_play_window#submit_button#misc#set_sensitive true ;
	 self#top_play_window#shuffle_button#misc#set_sensitive true ;
	 self#top_play_window#pause_button#misc#set_sensitive true ;
	 self#top_play_window#finish_button#misc#set_sensitive true ;
	 (* On rend la widget de saisie active par dfaut. *)
	 self#top_play_window#proposition_entry#misc#grab_focus () ;
	 (* On met une image de status d'attente. *)
	 self#top_play_window#status_pixmap#set_pixmap Global.waiting_pixmap ;
	 (* Rinitialisation de la progress bar. *)
	 self#top_play_window#elapsed_progressbar#configure
	   ~current: 0.0 ~min: 0.0
	   ~max: Global.current_prefs.Global.time_limit ;
	 (* Dclenchement du timeout de fin de jeu. *)
	 Ihm_misc.game_timeout :=
	   Some (GMain.Timeout.add
		   ~ms: 1000 ~callback: (Ihm_misc.check_end_of_game self))
	 end
     | None -> failwith "No selected dictionary."
    end



  method on_submit_button_clicked () =
    begin
    let word =
      String.uncapitalize (self#top_play_window#proposition_entry#text) in
    let add_score = Game.check_user_input word !Global.letters in
    if add_score = 0 then
      begin
      (* On met une image de status ngative. *)
      self#top_play_window#status_pixmap#set_pixmap Global.unhappy_pixmap
      end
    else
      begin
      (* On mmorise la nouvelle proposition accepte. *)
      Global.already_submitted := word :: !Global.already_submitted ;
      (* On met  jour la liste des solutions dj trouves. *)
      (match !Global.solutions_clist with
       | None -> assert false
       | Some clist -> ignore (clist#append [word])) ;
      incr Global.nb_found ;
      (* On incrmente la barre de progression de rsultats trouvs. *)
      self#top_play_window#player_progressbar#set_value
	(float_of_int !Global.nb_found) ;
      (* On met une image de status positive. *)
      self#top_play_window#status_pixmap#set_pixmap Global.happy_pixmap ;
      (* On met  jour le score. *)
      Global.score := !Global.score + add_score ;
      self#top_play_window#score_label#set_text
	(string_of_int !Global.score) ;
      (* On vide la widget de saisie. *)
      self#top_play_window#proposition_entry#set_text "" ;
      (* On vrifie si toutes les solutions ont t trouves. *)
      if (List.length !Global.already_submitted) >=
	!Global.nb_possibilities then
	self#on_finish_button_clicked ()
      else
	(* On rend la widget de saisie active par dfaut. *)
	self#top_play_window#proposition_entry#misc#grab_focus ()
      end
    end


  method on_proposition_entry_key_press_event ev =
    begin
    if GdkEvent.Key.keyval ev = GdkKeysyms._Return then
      self#on_submit_button_clicked () ;
    (* On retourne toujours false pour laisser le systme  *)
    (* finir d'appliquer les ventuels callbacks restants. *)
    false
    end



  method on_finish_button_clicked () =
    begin
    (* Arrter la minuterie. *)
    Ihm_misc.stop_timer () ;
    (* On dsactive les widgets de jeu. *)
    self#top_play_window#proposition_entry#misc#set_sensitive false ;
    self#top_play_window#submit_button#misc#set_sensitive false ;
    self#top_play_window#shuffle_button#misc#set_sensitive false ;
    self#top_play_window#pause_button#misc#set_sensitive false ;
    self#top_play_window#finish_button#misc#set_sensitive false ;
    (match !Global.solutions_clist with
     | None -> assert false
     | Some clist ->
	 (* On rajoute les solutions non trouves par le joueur en rouge. *)
	 let style = clist#misc#style#copy in
	 style#set_fg [(`NORMAL, `RGB (65535,0,0))] ;
	 List.iter
	   (fun sol ->
	     if not (List.mem sol !Global.already_submitted) then
	       let line = clist#append [sol] in
	       clist#set_cell ~style: style line 0)
	   !Global.possibilities) ;
    self#top_play_window#status_pixmap#set_pixmap Global.sleepy_pixmap ;
    let end_window = new Blabla_glade_interface.top_end_game_window self in
    self#set_end_game_window end_window ;
    end_window#score_label#set_text (string_of_int !Global.score) ;
    (* Lier la pression sur la touche 'Entre'  la sortie. *)
    ignore
      (end_window#end_game_window#event#connect#key_press
	 ~callback:
	 (fun ev ->
	   if (GdkEvent.Key.keyval ev = GdkKeysyms._Return) then
	     begin
	     self#on_close_end_game_window_button_clicked () ;
	     (* Retourner true pour ne pas laisser le gestionnaire   *)
	     (* par dfaut faire son travail et ainsi signifier que  *)
	     (* l'on a nous mme bien trait totalement l'vnement. *)
	     false
	     end else true)) ;
    let is_in_highscore =
      List.exists (fun (_, s, _) -> s < !Global.score) !Global.highscores in
    if is_in_highscore then
      begin
      end_window#score_name_entry#misc#grab_focus () ;      
      end_window#enter_name_hbox#misc#show ()
      end
    else end_window#enter_name_hbox#misc#hide () ;
    ignore (end_window#end_game_window#show ())
    end


  method on_close_end_game_window_button_clicked () =
    begin
    let player_name = self#top_end_game_window#score_name_entry#text in
    if player_name <> "" then
      begin
      (* Si le joueur a pu entrer son nom, c'est que la widget *)
      (* d'entre tait visible et donc qu'il tait admis dans *)
      (* les highscores. On va donc le mmoriser.              *)
      let newscores =
	(player_name, !Global.score, Global.current_prefs.Global.nb_letters) ::
	!Global.highscores in
      (* On re-trie al liste des scores par ordre dcroissant. *)
      let newscores = Sort.list (>) newscores in
      Global.highscores := Game.snap_list 10 newscores ;
      if not (Global.save_highscores ()) then
	Ihm_misc.error_notify self "Unable to save highscores."
      end ;
    (* On dtruit la fentre. *)
    self#top_end_game_window#end_game_window#destroy ()
    end


  method on_shuffle_button_clicked () =
    begin
    Game.shuffle_letters !Global.letters ;
    self#top_play_window#letters_label#set_text
      (Game.letters_to_string !Global.letters) ;
    (* On met une image de status d'attente. *)
    self#top_play_window#status_pixmap#set_pixmap Global.waiting_pixmap
    end



  method on_highscores_item_activate () =
    begin
    let highscores_window =
      new Blabla_glade_interface.top_highscores_window self in
    self#set_highscores_window highscores_window ;
    (* Ajout de la liste des scores  la main car par gre par MlGlade. *)
    let scores_scrolled_window =
      GBin.scrolled_window
	~hpolicy: `AUTOMATIC ~vpolicy: `AUTOMATIC
	~packing: (highscores_window#scores_list_vbox)#add () in
    let scores_wlist =
      GList.clist
	~packing: scores_scrolled_window#add ~titles_show: true ~width: 150
	~titles: ["Player" ; "Score"; "Letters"] () in
    scores_wlist#set_column ~justification: `CENTER ~resizeable: true 0 ;
    scores_wlist#set_column ~justification:`CENTER ~resizeable: true 1 ;
    List.iter
      (fun (name, score, nb_letters) ->
	ignore
	  (scores_wlist#append
	     [name ; (string_of_int score) ; (string_of_int nb_letters)]))
      !Global.highscores ;
    highscores_window#highscores_window#show () ;
    scores_wlist#columns_autosize ()
    end



  method on_close_highscore_button_clicked () =
    self#top_highscores_window#highscores_window#destroy ()


  method on_options_item_activate () =
    begin
    let options_window =
      new Blabla_glade_interface.top_options_window self in
    self#set_options_window options_window ;
    options_window#database_entry#set_text
      Global.current_prefs.Global.dictionary_filename ;
    options_window#number_letters_spinbutton#set_value
      (float_of_int Global.current_prefs.Global.nb_letters) ;
    options_window#ratio_hscale#adjustment#set_value
      (float_of_int Global.current_prefs.Global.ration_cons_voy) ;
    options_window#time_limit_spinbutton#set_value
      Global.current_prefs.Global.time_limit ;
    options_window#options_window#show ()
    end



  method on_save_options_button_clicked () =
    begin
    let options_window = self#top_options_window in
    (* On va commencer par essayer d'ouvrir le dictionnaire. *)
    (try
      let filename = options_window#database_entry#text in
      let dic_file = open_in_bin filename in
      let initial_base = ((input_value dic_file) : int) in
      let dictionary = ((input_value dic_file) : Common.dictionary) in
      close_in dic_file ;
      Global.dictionary := Some dictionary ;
      Global.initial_base := initial_base ;
      (* On initialise les prferences courantes avec le     *)
      (* dictionnaire seulement si on a russi  le charger. *)
      Global.current_prefs.Global.dictionary_filename <- filename
    with _ ->
      Ihm_misc.error_notify self
	"Unable to open dictionary.\nDictionary left unchanged.\nOther modification taken into account.") ;
    Global.current_prefs.Global.nb_letters <-
      truncate options_window#number_letters_spinbutton#value ;
    Global.current_prefs.Global.ration_cons_voy <-
      truncate options_window#number_letters_spinbutton#value ;
    Global.current_prefs.Global.time_limit <-
      float_of_int (truncate (options_window#ratio_hscale#adjustment#value)) ;
    Global.current_prefs.Global.time_limit <-
      options_window#time_limit_spinbutton#value ;
    self#top_options_window#options_window#destroy ()
    end


  method on_forget_options_button_clicked () =
    self#top_options_window#options_window#destroy ()


  method on_window_error_close_button_clicked () =
    self#top_window_error#window_error#destroy ()


  method on_about_activate () =
    begin
    let about_window =
      new Blabla_glade_interface.top_about_window self in
    self#set_about_window about_window ;
    about_window#about_close_button#misc#grab_focus () ;      
    about_window#about_window#misc#show ()
    end


  method on_about_close_button_clicked () =
    self#top_about_window#about_window#destroy ()
end
