let snap_list len l =
  let rec rec_snap = function
    | (0, _) -> []
    | (n, []) ->  []
    | (n, (h :: q)) -> h :: (rec_snap ((n -1), q)) in
  rec_snap (len, l)
;;



let append_unique data l =
  let rec rec_append = function
    | [] -> [data]
    | h :: q ->
        h :: (if h = data then q else (rec_append q)) in
  rec_append l
;;



(* Fonction qui recopie un tableau sauf l'lement dsign par l'index. *)
let copy_but src dest index =
  let dest_index = ref 0 in
  Array.iteri
    (fun i ch ->
      if i <> index then
        begin
          dest.(!dest_index) <- ch ;
          incr dest_index
        end)
    src
;;



let modify_word_for_game w =
  let len = String.length w in
  if len < 3 then None
  else
    begin
      let w' = String.copy w in
      for i = 0 to len - 1 do
        match w'.[i] with
        | '' | '' | '' -> w'.[i] <- 'a'
        | '' -> w'.[i] <- 'c'
        | '' | '' | '' | '' -> w'.[i] <- 'e'
        | '' | '' -> w'.[i] <- 'i'
        | '' -> w'.[i] <- 'u'
        | _ -> ()
      done ;
      Some w'
    end
;;




(* Fonction qui recherche tous les mots possibles faisables par une *)
(* combinaison des lettres utilises au plus une fois.              *)
let combine dictionary initial_base initial_letters =
  let found_words = ref ([] : string list) in
  let rec rec_combine letters base prefix =
    let len_letters = Array.length letters in
    if base <> Common.end_transition && len_letters > 0 then
      begin
        let len_letters_minus_one = len_letters - 1 in
        (* Tableau avec 1 lettre de moins. *)
        let letters' = Array.make len_letters_minus_one ' ' in
        for range = 0 to len_letters_minus_one do
          (* Code ASCII de la lettre choisie. *)
          let i = Char.code letters.(range) in
          if (base + i < dictionary.Common.size) then
            begin
              let tmp_char =
                Common.get_char dictionary.Common.elements.(base + i) in
              (* On vrifie qu'il y a bien transition, et donc que le champ *)
              (* est utilis.                                               *)
              if tmp_char = i then
                begin
                  let prefix' = prefix ^ (Char.escaped (Char.chr tmp_char)) in
                  (* On vrifie si le dagelement est final. *)
                  if Common.is_final dictionary.Common.elements.(base + i) then
                    begin
                      let final_word = modify_word_for_game prefix' in
                      match final_word with
                      | None -> ()
                      | Some w-> found_words := append_unique w !found_words
                    end ;
                  copy_but letters letters' range ;
                  rec_combine letters'
                    (Common.get_next_base dictionary.Common.elements.(base + i))
                    prefix'
                end      (* Fin de if tmp_char = i *)
            end        (* Fin de if (base + i < dictionary.Common.size) *)
        done           (* Fin de for range = 1 to ... do *)
      end in         (* Fin de if base <> Common.end_transition && *)
  rec_combine initial_letters initial_base "" ;
  !found_words
;;



(* let any_letter = [|
   'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';'n';'o';'p';'q';'r';'s';'t';'u';'v';'w';'x';'y';'z'
   |] ;; *)
let any_voyelle = [|'a';'e';'i';'o';'u';'y'|] ;;
let any_consonne = [|
  'b';'c';'d';'f';'g';'h';'j';'k';'l';'m';'n';'p';'q';'r';'s';'t';'v';'w';'x';'z'
|] ;;

(* let any_letter_len = Array.length any_letter ;; *)
let any_voyelle_len = Array.length any_voyelle ;;
let any_consonne_len = Array.length any_consonne ;;


let round fl =
  let below = int_of_float fl in
  let above = float (below + 1) in
  if fl +. 0.5 >= above then int_of_float above
  else below
;;



(* ********************************************************
   [Description]: Choisit un ensemble de lettre autorises.
 ******************************************************** *)
let rec choose_letters nb ratio_cons_voy =
  if ratio_cons_voy <= 0 || ratio_cons_voy > 100 then
    (* Choix non contraint. *)
    Array.init nb (fun _ -> Char.chr ((Random.int 26) + Char.code 'a'))
  else
    begin
      (* Choix des consonnes. Entre ASCII [] *)
      let nb_consonnes =
        round (((float nb) *. (float ratio_cons_voy)) /. 100.0) in
      (* Choix des voyelles. Entre ASCII [] *)
      let nb_voyelles = nb - nb_consonnes in
      let res = Array.create nb ' ' in
      Array.iteri
        (fun index _ ->
          if index < nb_consonnes then
            res.(index) <- any_consonne.(Random.int any_consonne_len)
          else res.(index) <- any_voyelle.(Random.int any_voyelle_len))
        res ;
      res
    end
;;



(* Fonction qui transforme une liste de caractres en chane de caractres *)
(* en mettant quelques espaces enrtre chaque caractre pour arrer un peu. *)
let letters_to_string letters =
  let str = ref "" in
  Array.iter (fun l -> str := !str ^ "  " ^ Char.escaped l) letters ;
  !str
;;



(* Fonction qui mlange les lmnts d'un tableau. *)
let shuffle_letters letters =
  let len_minus_one = Array.length letters - 1 in
  for i = 0 to len_minus_one do
    let i = Random.int len_minus_one in
    let j = Random.int len_minus_one in
    let t = letters.(i) in
    letters.(i) <- letters.(j) ;
    letters.(j) <- t
  done
;;




(* *********************************************************************
   [Description]: Vrifie qu'une lettre appartient bien  l'ensemble des
   lettres autorises et retourne le nouvel ensemble de lettres
   autorises dans lequel la letrre vrifi a t soustraite.
 ********************************************************************* *)
let check_letter_belonging letter among =
  let rec rec_check = function
    | [] -> (false, [])
    | l :: q ->
        if l = letter then (true, q)
        else
          let (answer, remain) = rec_check q in
          (answer, l :: remain) in
  rec_check among
;;




(* **************************************************************
   [Description]: Vrifie qu'un mot est bien form uniquement des
   lettres autorises.
 ************************************************************** *)
let check_word_belonging word initial_among =
  let rec rec_check among = function
    | 0 -> fst (check_letter_belonging word.[0] among)
    | n ->
        match check_letter_belonging word.[n] among with
        | (false, _) -> false
        | (true, rem) -> rec_check rem (n -1) in
  rec_check initial_among ((String.length word) - 1)
;;


(* *************************************************************
   [Description]: Vrifie qu'un mot est une solution acceptable.
 ************************************************************* *)
let check_user_input word authorized =
  (* On doit vrifier que les lettres utilises appartiennent bien *)
  (* aux lettres utilises et qu'elles ne sont pas utilises en    *)
  (* double. Puis on doit vrifier que le mot existe bien dans le  *)
  (* dictionnaire.                                                 *)
  if (String.length word) = 0 then 0
  else
    begin
      if List.mem word !Global.already_submitted then
        0                        (* Mot dj propos. *)
      else
        begin
          if check_word_belonging word (Array.to_list authorized) then
            begin
              if List.mem word !Global.possibilities then
                String.length word   (* Mot correct et non dj propos. *)
              else 0                 (* Mot non prsent dans le dictionnaire. *)
            end
          else 0                   (* Lettres non autorises. *)
        end
    end
;;
