type preferences = {
  mutable dictionary_filename : string ;
  mutable nb_letters : int ;
  mutable ration_cons_voy : int ;
  mutable time_limit : float
} ;;


let current_prefs = {
  dictionary_filename = "" ;
  nb_letters = 6 ;
  ration_cons_voy = 33 ;
  time_limit = 60.0
} ;;


let user = ref "" ;;
let score = ref 0 ;;
let possibilities = ref ([] : string list) ;;
let nb_possibilities = ref 0 ;;
let already_submitted = ref ([] : string list) ;;
let nb_found = ref 0 ;;
let initial_base = ref 0 ;;
let dictionary = ref (None : Common.dictionary option) ;;
let letters = ref ([||] : char array) ;;
let solutions_clist = ref (None : (string GList.clist) option) ;;
let happy_pixmap =
  GDraw.pixmap_from_xpm
    ~file: "/export/devel/francois/devel/merime2_0_1/blabla/happy.xpm" () ;;
let unhappy_pixmap =
  GDraw.pixmap_from_xpm
    ~file: "/export/devel/francois/devel/merime2_0_1/blabla/unhappy.xpm" () ;;
let sleepy_pixmap =
  GDraw.pixmap_from_xpm
    ~file: "/export/devel/francois/devel/merime2_0_1/blabla/sleepy.xpm" () ;;
let waiting_pixmap =
  GDraw.pixmap_from_xpm
    ~file: "/export/devel/francois/devel/merime2_0_1/blabla/waiting.xpm" () ;;
let highscores = ref [ ("", 0, 0) ; ("", 0, 0) ; ("", 0, 0) ; ("", 0, 0) ;
                       ("", 0, 0) ; 
                       ("", 0, 0) ; ("", 0, 0) ; ("", 0, 0) ; ("", 0, 0) ;
		               ("", 0, 0) ] ;;


(* *********************************************************************
   [Exception] No_home_dir
   [Description] : Sert  signaler qu'il est impossible de dterminer le
   homedir de l'utilisateur.
   [Rem] : Exporte hors de ce module.
 ********************************************************************* *)
exception No_home_dir ;;



(* *********************************************************************
   [Fonc] : gethomedir : unit -> string
   [Description] : Fonction charge de retrouver le home directory de
   l'utilisateur. On commence d'abord par regarder la variable
   d'environnement '$HOME'. Si on ne le trouve pas, on va rechercher
   l'uid de l'utilisateur et on va parser le fichier '/etc/passwd'
   pour retrouver le champ homedir correspondant.
   [Args] :
   - Nant
   [Retour] :
   - string : La chane de caractres reprsentant le chemin complet
   du homedir de l'utilisateur.
   [Rem] : Exporte hors de ce module.
   Peut lever l'exception No_home_dir si l'on n'a pas russi 
   dterminer le homedir de l'utilisateur, ni  partir de la
   variable d'environnement, ni  partir du '/etc/passwd'.
 ********************************************************************* *)
let gethomedir () =
  try Sys.getenv "HOME" with
  | Not_found ->
      begin
        let uid = Unix.getuid () in
        let etc_passwd = try open_in "/etc/passwd" with _ -> raise No_home_dir in
        let home_dir = ref "" in
        try
	      while !home_dir = "" do
	        let line = input_line etc_passwd in
	        (* Saute au password. *)
	        let i = String.index line ':' in
	        (* Saute au dbut de l'uid. *)
	        let i' = String.index_from line (i + 1) ':' in
	        (* Saute au dbut du gid *)
	        let i'' = String.index_from line (i' + 1) ':' in
	        let id_string = String.sub line (i' + 1) (i'' - i' - 1) in
	        let id = int_of_string id_string in
	        if id = uid then
	          begin
                (* Saute  la fin du gid. *)
	            let i3 = String.index_from line (i'' + 1) ':' in
                (* Saute au dbut du homedir. *)
	            let i4 = String.index_from line (i3 + 1) ':' in
                (* Fin du homedir. *)
	            let i5 = String.index_from line (i4 + 1) ':' in
	            home_dir := String.sub line (i4 + 1) (i5 - i4 - 1) ;
	          end
	      done ;
	      close_in etc_passwd ;
	      !home_dir
        with _ -> raise No_home_dir
      end
;;




let load_highscores () =
  try
    let home_dir = gethomedir () in
    let file_name = home_dir ^ "/" ^ ".blabla_highscores" in
    if Sys.file_exists file_name then
      begin
        let in_hd = open_in_bin file_name in
        highscores := ((input_value in_hd) : ((string * int * int) list)) ;
        close_in in_hd ;
      end ;
    true
  with No_home_dir -> false
;;



let save_highscores () =
  try
    let home_dir = gethomedir () in
    let file_name = home_dir ^ "/" ^ ".blabla_highscores" in
    let out_hd = open_out_bin file_name in
    output_value out_hd !highscores ;
    close_out out_hd ;
    true
  with No_home_dir -> false
;;

