let game_timeout = ref (None : GMain.Timeout.id option) ;;


let stop_timer () =
  match !game_timeout with
  | None -> ()
  | Some id ->
      GMain.Timeout.remove id ;
      game_timeout := None
;;


let check_end_of_game callback_object () =
  begin
    let elapsed_pbar = callback_object#top_play_window#elapsed_progressbar in
    let elapsed_time = elapsed_pbar#value in
    if elapsed_time < Global.current_prefs.Global.time_limit then
      elapsed_pbar#set_value (elapsed_time +. 1.0)
    else
      begin
        (* Fin du temps rglementaire. *)
        stop_timer () ;
        callback_object#on_finish_button_clicked ()
      end ;
    true
  end
;;


(* *************************************************************************
   [Fonc] error_notify :
   < on_window_error_close_button_clicked : unit -> unit;
   set_window_error :
   Gui_glade_interface.top_window_error -> unit;
   .. > ->
   string -> unit
   [Description] : Affichage d'une fentre d'erreur modale au dessus de la
   fentre principale retrouve via le 'callback_object' pass en
   argument. Un icone d'erreur est affich  gauche de cette fentre.
   [Args] :
   - callback_object : L'objet provenant de Gui_mine.all_callbacks et
   mmorisant le contexte 'Glade' de l'interface graphique.
   - string : Le message d'erreur  afficher dans la fentre.
   [Retour] :
   - unit.
   [Rem] : Exporte hors de ce module.
 ************************************************************************* *)
let error_notify callback_object msg =
  let window_error =
    new Blabla_glade_interface.top_window_error callback_object in
  callback_object#set_window_error window_error ;
  window_error#error_label#set_text msg ;
  ignore
    (GtkBase.Widget.add_events 
       window_error#window_error#as_widget [`ALL_EVENTS]) ;
  window_error#window_error#set_position `CENTER ;
  window_error#window_error#show () ;
  (* Lier la pression sur la touche 'Entre'  la sortie. *)
  ignore
    (window_error#window_error#event#connect#key_press
       ~callback:
       (fun ev ->
         if (GdkEvent.Key.keyval ev = GdkKeysyms._Return) then
           begin
             window_error#window_error#destroy () ;
             (* Retourner true pour ne pas laisser le gestionnaire   *)
             (* par dfaut faire son travail et ainsi signifier que  *)
             (* l'on a nous mme bien trait totalement l'vnement. *)
             false
           end else true)) ;
  (* Lier la pression sur la touche 'Escape'  la sortie. *)
  ignore
    (window_error#window_error#event#connect#key_press
       ~callback:
       (fun ev ->
         if (GdkEvent.Key.keyval ev = GdkKeysyms._Escape) then
           begin
             window_error#window_error#destroy () ;
             (* Retourner true pour ne pas laisser le gestionnaire   *)
             (* par dfaut faire son travail et ainsi signifier que  *)
             (* l'on a nous mme bien trait totalement l'vnement. *)
             false
           end else true)) ;
  ignore
    (window_error#window_error#connect#destroy ~callback: GMain.Main.quit) ;
  GMain.Main.main ()
;;
