
let fitting dic_buckets base dagelems nb_dag_elems =
  let fit = ref true in
  let i = ref 0 in
  while !i < nb_dag_elems && !fit = true
  do
    let dagelem_char = Common.get_char dagelems.(!i) in
    let dic_char = Common.get_char dic_buckets.(base + dagelem_char) in
    if dic_char <> 0 then fit := false ;
    incr i
  done ;
  !fit
;;



let suffixes =
  ((Hashtbl.create 13) : (Common.bucket array, Common.base) Hashtbl.t) ;;


let make_states =
  let nb_nodes = ref 0 in
  let base_start = ref 0 in

  fun dictionary stack_buckets nb_stack_buckets ->
    (* Non-random first fit. *)
    if !nb_nodes >= 500 then
      begin
        nb_nodes := 0 ;
        base_start := !base_start + 500 ;
      end ;

    let go_on = ref true in
    let base = ref !base_start in

    (* We look if we didn't already create a state of this form. *)
    try Hashtbl.find suffixes stack_buckets with Not_found ->
      begin
        while !go_on do
          (* We first ensure that the dictionary is not too small. *)
          if !base + Common.get_char stack_buckets.(nb_stack_buckets - 1) >=
            dictionary.Common.size then
            begin
              (* This won't fit. We must reallocate memory. *)
              let new_dic_size = dictionary.Common.size + 16000 in
              let new_dic_elems = Array.create new_dic_size Int32.zero in
              Array.blit
                dictionary.Common.elements 0 new_dic_elems 0
                dictionary.Common.size ;
              dictionary.Common.size <- new_dic_size ;
              dictionary.Common.elements <- new_dic_elems
            end
          else
	        begin
	          (* A bit of common sub-expressions sharing... *)
	          let dic_buckets = dictionary.Common.elements in
              (* We don't exceed the memory zone limit. *)
	          if not (Common.is_base dic_buckets.(!base)) then
                (* It's a good thing, the base is not yet used. *)
	            begin
                  (* We test that characters fit. *)
	              let fit =
	                fitting dic_buckets !base stack_buckets nb_stack_buckets in
	              if fit then
	                begin
                      (* We found a location where all fits. *)
	                  for i = 0 to nb_stack_buckets - 1 do
		                let tmp_char = Common.get_char stack_buckets.(i) in
	                    (* We only leave the bit indicating if the cell is a
                           base. *)
		                dic_buckets.(!base + tmp_char) <-
		                  Int32.logand
                            dic_buckets.(!base + tmp_char)
                            (Int32.of_string "0x00000100") ;
	                    (* We remove the bit used to tell if the field
                           NEXTBASE is set. *)
		                dic_buckets.(!base + tmp_char) <-
                          Int32.logor
		                    dic_buckets.(!base + tmp_char)
		                    (Int32.logand
                               stack_buckets.(i)
                               (Int32.of_string "0xFFFFFEFF")) (* ~0x00000100 *)
	                  done ;
	                  Common.set_is_base dic_buckets !base ;
	                  nb_nodes := !nb_nodes + nb_stack_buckets ;
	                  go_on := false                 (* We have finished. *)
	                end
	              else
	                incr base
	            end
	          else
	            incr base
	        end
        done ;
        (* Ajouter l'tat dans la table des tats dj crs. *)
        Hashtbl.add suffixes stack_buckets !base ;
        !base
      end
;;
