


let find_diff_index s1 s2 =
  let len_min = min (String.length s1) (String.length s2) in
  let i = ref 0 in
  while !i < len_min && s1.[!i] = s2.[!i] do incr i done ;
  !i
;;



let make_dictionary in_file dictionary =
  let stack = ((Array.create 500 Int32.zero) : Common.bucket array) in
  let sp = ref 0 in
  let current_word = ref "" in

  try
    while true do
      let old_word = !current_word in
      (* Read a new word in the input file. *)
      current_word := input_line in_file ;
      (* idiff points on the character making the first difference between the
         2 words. Hence, it is equal to the number of identical characters. *)
      let idiff = find_diff_index !current_word old_word in

      let creation_depth = (String.length old_word) - idiff - 1 in
      (* If creation_depth < 0 then there is no state to create and no
         END_TRANSITION to add to the last state of the stack.
         If creation_depth = 0 then there is only a END_TRANSITION to add on the
         last state of the stack.
         If creation_depth > 0 then there is a END_TRANSITION to add on the
         last state of the stack, and creation_depth layers to create in the
         dag.
         REM : for optimisation sake, we group creation_depth > 0 and
         creation_depth = 0 into creation_depth >= 0. *)
      if creation_depth >= 0 then
	    begin
	      Common.set_next_base stack !sp Common.end_transition ;
	      Common.set_next_filled stack !sp ;

          (* We will create creation_depth layers in the tree. *)
	      for j = 0 to creation_depth - 1 do
	        (* We will group all the transitions of this layer. *)
	        let nb_transitions = ref 0 in
	        while Common.is_next_filled stack.(!sp) do
	          (* This bucket's field NEXTBASE is set, hence it will virtually be
	             popped and transmitted to the routine creating
                 states / transitions. *)
	          incr nb_transitions ;
	          decr sp
	        done ;
	        let sub_stack = Array.sub stack (!sp + 1) !nb_transitions in
	        let tmp_base =
	          States.make_states dictionary sub_stack !nb_transitions in
	        Common.set_next_base stack !sp tmp_base ;
	        Common.set_next_filled stack !sp ;
	      done
	    end ;

      (* Now we will push the new letters of the new word. *)
      for j = idiff to (String.length !current_word) - 1 do
	    incr sp ;
	    Common.clear_elem stack !sp ;
	    Common.set_char stack !sp !current_word.[j]
      done ;
      Common.set_is_final stack !sp
    done ;
    assert false

  with End_of_file ->
    (* We have no more word to input, hence we will empty the stack. *)
    let old_word = !current_word in
    let creation_depth = String.length old_word in
    Common.set_next_base stack !sp Common.end_transition ;
    Common.set_next_filled stack !sp ;
    
    for j = 0 to creation_depth - 1 do
      (* We group all the transitions of the layer. *)
      let nb_transitions = ref 0 in
      while Common.is_next_filled stack.(!sp) do
        (* This bucket's field NEXTBASE is set so it will virtually be popped
           and transmitted to the routine creating states / transitions. *)
        incr nb_transitions ;
        decr sp
      done ;
      let sub_stack = Array.sub stack (!sp + 1) !nb_transitions in
      let tmp_base = States.make_states dictionary sub_stack !nb_transitions in
      Common.set_next_base stack !sp tmp_base ;
      Common.set_next_filled stack !sp
    done ;

    Common.get_next_base stack.(!sp)
;;
