(*
  Bits 0-7 de poids faible = Char
  Bit 8 = IsBase | NextFilled
  Bit 9 = IsFinal
  Bits 10-30 = NextBase
 *)
type bucket = Int32.t ;;
type base = int ;;

type dictionary = {
    mutable size : int ;
    mutable elements : bucket array
  } ;;

let end_transition = (0x1FFFFF : base) ;; (* 0x3FFFFF sur 32 bits *)

let clear_elem tbl index = tbl.(index) <- Int32.zero ;;

let get_char dagelem =
  Int32.to_int (Int32.logand dagelem (Int32.of_string "0xFF")) ;;

let is_base dagelem =
  (Int32.logand dagelem (Int32.of_string "0x00000100")) <> Int32.zero ;;

let is_next_filled dagelem =
  (Int32.logand dagelem (Int32.of_string "0x00000100")) <> Int32.zero ;;

let is_final dagelem =
  (Int32.logand dagelem (Int32.of_string "0x00000200")) <> Int32.zero ;;

let get_next_base dagelem =
  Int32.to_int (Int32.shift_right_logical dagelem 10)
;;

let set_char tbl index char =
  tbl.(index) <- Int32.logand tbl.(index) (Int32.of_string "0xFFFFFF00") ;
  tbl.(index) <- Int32.logor tbl.(index) (Int32.of_int (int_of_char char))
;;

let set_is_base tbl index =
  (* 1 << 8 *)
  tbl.(index) <- Int32.logor tbl.(index) (Int32.of_string "0x100") ;;

let set_next_filled tbl index =
  (* 1 << 8 *)
  tbl.(index) <- Int32.logor tbl.(index) (Int32.of_string "0x100") ;;

let set_is_final tbl index =
  (* 1 << 9 *)
  tbl.(index) <- Int32.logor tbl.(index) (Int32.of_string "0x200") ;;

let set_next_base tbl index base_value =
  tbl.(index) <- Int32.logand tbl.(index) (Int32.of_string "0x000003FF") ;
  tbl.(index) <- Int32.logor tbl.(index) (Int32.of_int (base_value lsl 10))
;;
