

let find_words dictionary initial_base pattern =
  let dic_size = dictionary.Common.size in
  let dic_buckets = dictionary.Common.elements in
  let pattern_length = String.length pattern in  

  let rec rec_find base pattern_index prefix =
    if base <> Common.end_transition then
      begin
        (* We ensure that we don't run outside the pattern while processing. *)
        if pattern_index < pattern_length then
          begin
            let (start_letter, stop_letter) =
              (match pattern.[pattern_index] with
              | '*' -> (1, 255)
              | whatever -> let lc = Char.code whatever in (lc, lc)) in
            for letter_code = start_letter to stop_letter do
              if base + letter_code < dic_size then
                begin
                  (* We check that a transition exists. *)
                  if Common.get_char dic_buckets.(base + letter_code) =
                    letter_code then
                    begin
                      let prefix' =
                        prefix ^ (Char.escaped (Char.chr letter_code)) in
                      (* We check if we arrived at the end of the pattern
                         to match. *)
                      if pattern_index = pattern_length - 1 then
                        begin
                          if Common.is_final
                              dic_buckets.(base + letter_code) then
                            (* The found solution is a complete word of the
                               dictionary. *)
                            Printf.printf "-> %s\n" prefix'
                        end
                      else
                        (* We didn't arrive to the end of the pattern, then we
                           must continue. *)
                        rec_find
                          (Common.get_next_base
                             dic_buckets.(base + letter_code))
                          (pattern_index + 1) prefix'
                    end
                end
            done
          end
      end in
  rec_find initial_base 0 ""
;;
