

let display_dictionary dictionary initial_base =
  let rec rec_display base prefix =
    if base <> Common.end_transition then
      begin
        for i = 1 to 255 do
          if (base + i < dictionary.Common.size) then
            begin
              let tmp_char =
                Common.get_char dictionary.Common.elements.(base + i) in
              (* We ensure that there really is a transition, and hence that the
                 field is used. *)
              if tmp_char = i then
                begin
                  let prefix' = prefix ^ (Char.escaped (Char.chr tmp_char)) in
                  (* We check if the dagelement is final. *)
                  if Common.is_final dictionary.Common.elements.(base + i)
                  then Printf.printf "%s\n" prefix' ;
                  rec_display
                    (Common.get_next_base dictionary.Common.elements.(base + i))
                    prefix'
                end
            end
        done
      end in
  rec_display initial_base ""
;;


let raw_dump dictionary initial_base =
  Printf.printf "Initial base: %d\n" initial_base ;
  Printf.printf "********************************************************\n" ;
  let dic_elems = dictionary.Common.elements in
  for i = 0 to dictionary.Common.size - 1 do
    Printf.printf "%s" (if i = initial_base then "!  " else "   ") ;
    Printf.printf "%d :  " i ;
    Printf.printf "Elem 0x%s  " (Int32.to_string dic_elems.(i)) ;
    Printf.printf "Char '%s'  "
      (Char.escaped
         (Char.chr (Common.get_char dic_elems.(i)))) ;
    let nb = Common.get_next_base dic_elems.(i) in
    Printf.printf " -> %d (0x%x)   " nb nb ;
    if Common.is_final dic_elems.(i) then Printf.printf "F" ;
    if Common.is_base dic_elems.(i) then Printf.printf "B" ;
    Printf.printf "\n" ;
  done
;;


let int32_to_4_bytes i32 =
  let byte_mask = Int32.of_int 255 in
  let b0 =
    Int32.to_int (Int32.logand (Int32.shift_right_logical i32 24) byte_mask) in
  let b1 =
    Int32.to_int (Int32.logand (Int32.shift_right_logical i32 16) byte_mask) in
  let b2 =
    Int32.to_int (Int32.logand (Int32.shift_right_logical i32 8) byte_mask) in
  let b3 = Int32.to_int (Int32.logand i32 byte_mask) in
  (b0, b1, b2, b3)
;;


(* abcd --> in the file: dcba *)
(* b0, b1, b2, b3 *)
let raw_binary_dump dictionary initial_base out_filename =
  let out_channel = open_out_bin out_filename in
  (* Outputs the magic number *)
  let (b0, b1, b2, b3) = ((Char.code 'M'), 
                          (Char.code 'D'),
                          (Char.code 'I'),
                          (Char.code 'C')) in
  output_byte out_channel b3 ;
  output_byte out_channel b2 ;
  output_byte out_channel b1 ;
  output_byte out_channel b0 ;
  (* Outputs the size of the array representing the DAG in number of cells. *)
  let (b0, b1, b2, b3) =
    int32_to_4_bytes (Int32.of_int dictionary.Common.size) in
  output_byte out_channel b3 ;
  output_byte out_channel b2 ;
  output_byte out_channel b1 ;
  output_byte out_channel b0 ;
  (* Outputs the initial base *)
  let (b0, b1, b2, b3) = int32_to_4_bytes (Int32.of_int initial_base) in
  output_byte out_channel b3 ;
  output_byte out_channel b2 ;
  output_byte out_channel b1 ;
  output_byte out_channel b0 ;
  (* Outputs each of the elements of the DAG. *)
  let dic_elems = dictionary.Common.elements in
  for i = 0 to dictionary.Common.size - 1 do
    let (b0, b1, b2, b3) = int32_to_4_bytes dic_elems.(i) in
    output_byte out_channel b3 ;
    output_byte out_channel b2 ;
    output_byte out_channel b1 ;
    output_byte out_channel b0
  done ;
  close_out out_channel
;;
