(* Loads the personnal dictionary of the user if it exist otherwise does
   nothing. Its name is global in 'Spellglobal.private_filename'. *)
let load_private_dictionary () =
  try
    let file = open_in !Spellglobal.private_filename in
    let go_on = ref true in
    while !go_on do
      try let w = input_line file in
      Spellglobal.private_auxdic :=
        Spellglobal.Auxdic.add w !Spellglobal.private_auxdic
      with End_of_file -> go_on := false
    done ;
    close_in file
  with Sys_error _ -> ()
;;



(* Save the personnal dictionary of the user if it was modified.
   Proposes to save it directly, to change its name or not to save it. *)
let save_private_dictionary () =
  (* Try to save the dictionary under the current global name. *)
  let try_save () =
    let go_on = ref true in
    while !go_on do
      try
	    let file = open_out !Spellglobal.private_filename in
	    Spellglobal.Auxdic.iter (fun w -> output_string file (w ^ "\n"))
	      !Spellglobal.private_auxdic ;
	    close_out file ;
	    go_on := false
      with Sys_error _ ->
	    Printf.printf "Cannot write private dictionary.\n" ;
	    Printf.printf "Enter a filename or ^C to abort.\n" ;
	    Spellglobal.private_filename := read_line ()
    done in
  (* Ask if the user wants to save the dictionary. *)
  if !Spellglobal.private_modified then
    begin
      let go_on = ref true in
      while !go_on do
        Printf.printf "Private dictionary was modified.\n" ;
        Printf.printf "Save it under '%s' (yes, no or new) ?\n"
          !Spellglobal.private_filename ;
        flush stdout ;
        match read_line () with
        | "no" -> go_on := false
        | "yes" -> go_on := false ; try_save ()
        | "new" ->
            go_on := false ;
	        Printf.printf "Enter a filename\n" ;
	        Spellglobal.private_filename := read_line () ;
	        try_save ()
	    | _ -> Printf.printf "Please answer accurately.\n"
      done
    end
;;
