type operation = Nop | Suppr | Add | Wrong ;;

module Suggs = Map.Make (struct type t = string let compare = compare end) ;;
let suggestions = ref Suggs.empty ;;


let add_suggestion suggestion weight =
  try
    let old_sug_weight = Suggs.find suggestion !suggestions in
    if old_sug_weight > weight then
      suggestions := Suggs.add suggestion weight
          (Suggs.remove suggestion !suggestions)
  with
  | Not_found -> suggestions := Suggs.add suggestion weight !suggestions
;;



let suggest dictionary initial_base word_position wrong_word =
  let len_wword = String.length wrong_word in
  let num_allowed_errors = if len_wword >= 7 then 2 else 1 in
  let dic_size = dictionary.Common.size in
  let dic_elems = dictionary.Common.elements in
  (* Remove all the previous suggestions. *)
  suggestions := Suggs.empty ;

  let permutations () =
    for i = 0 to len_wword - 2 do
      let word' = String.copy wrong_word in
      let tmp = word'.[i] in
      word'.[i] <- word'.[i+1] ;
      word'.[i+1] <- tmp ;
      if Exist.exists dictionary initial_base word' then
        add_suggestion word' 1
    done in

  let rec rec_suggest base last_op index_wword suggestion was_final err_cnt =
    if err_cnt <= num_allowed_errors then
      begin
        if base = Common.end_transition || was_final then
          begin
            (* Le nombre rel d'erreur doit prendre en compte la diffrence de
               longueur entre la mot d'origine et l'endroit o on en est dedans.
               S'il, par rapport  cet endroit, il reste encore 2 lettres dans
               le mot d'origine, a correspond en fait  une suggestion trouve
               avec 2 oublis en plus des erreurs dj accumules. Attention,
               l'index courant rfre  une lettre qu'on n'a pas encore traite,
               donc le nombre d'erreurs rel est :
                 (len_wword - 1 - index_wword + err_cnt) + 1
               On abbrge un peu en virant - 1 + 1. *)
            let real_err_cnt = len_wword - index_wword + err_cnt in
            if real_err_cnt <= num_allowed_errors then
              add_suggestion suggestion real_err_cnt
          end ;

        if base <> Common.end_transition then
          begin
            if index_wword < len_wword then
              begin
                (* Considrons que la lettre courante est correcte et que la
                   lettre courante est fausse (factorisation). *)
                let good_letter_code = Char.code wrong_word.[index_wword] in
                for letter_code = 1 to 255 do
                  if base + letter_code < dic_size &&
                    Common.get_char
                      dic_elems.(base + letter_code) = letter_code then
                    begin
                      (* Partage de sous-expression commune. *)
                      let new_bucket = dic_elems.(base + letter_code) in
                      let new_base = Common.get_next_base new_bucket in
                      let is_final = Common.is_final new_bucket in
                      (* On change juste le poids et la dernire opration
                         effectue en fonction de si on est dans le cas o la
                         lettre est correcte ou fausse. *)
                      if letter_code = good_letter_code then
                        rec_suggest new_base Nop (index_wword + 1)
                          (suggestion ^ (Char.escaped (Char.chr letter_code)))
                          is_final err_cnt
                      else
                        rec_suggest new_base Wrong (index_wword + 1)
                          (suggestion ^ (Char.escaped (Char.chr letter_code)))
                          is_final (err_cnt + 1)
                    end
                done ;
                (* Considrons qu'il y a une lettre en trop dans le mot *)
                if last_op <> Add then
                  rec_suggest base Suppr (index_wword + 1)
                    suggestion false (err_cnt + 1)
              end ;
            (* Considrons qu'il manque une lettre dans le mot. *)
            if last_op <> Suppr then
              for letter_code = 1 to 255 do
                let letter = Char.chr letter_code in
                if base + letter_code < dic_size &&
                  Common.get_char
                    dic_elems.(base + letter_code) = letter_code then
                  begin
                    (* Partage de sous-expression commune. *)
                    let new_bucket = dic_elems.(base + letter_code) in
                    let new_base = Common.get_next_base new_bucket in
                    let is_final = Common.is_final new_bucket in
                    rec_suggest new_base Add index_wword
                      (suggestion ^ (Char.escaped letter))
                      is_final (err_cnt + 1)
                  end
              done
          end
      end
  in
  rec_suggest initial_base Nop 0 "" false 0 ;
  permutations ()
;;

