(* Function that must find the user's home directory.
   We search its uid and we parse the /etc/passwd in order to recover the
   corresponding homedir field. *)
exception No_home_dir ;;

let gethomedir () =
  let uid = Unix.getuid () in
  let etc_passwd = try open_in "/etc/passwd" with _ -> raise No_home_dir in
  let home_dir = ref "" in
  try
    while !home_dir = "" do
      let line = input_line etc_passwd in
      (* Jump to the password. *)
      let i = String.index line ':' in
      (* Jump to the beginning of the uid. *)
      let i' = String.index_from line (i + 1) ':' in
      (* Jump to the beginning of the gid. *)
      let i'' = String.index_from line (i' + 1) ':' in
      let id_string = String.sub line (i' + 1) (i'' - i' - 1) in
      let id = int_of_string id_string in
      if id = uid then
        begin
          (* Jump to the end of the gid. *)
          let i3 = String.index_from line (i'' + 1) ':' in
          (* Jump to the beginning of the homedir. *)
          let i4 = String.index_from line (i3 + 1) ':' in
          (* End of the homedir. *)
          let i5 = String.index_from line (i4 + 1) ':' in
          home_dir := String.sub line (i4 + 1) (i5 - i4 - 1) ;
        end
    done ;
    close_in etc_passwd ;
    !home_dir
  with _ -> raise No_home_dir
;;

