let dic_name = ref "" ;;
let in_filename = ref "" ;;
let echo_on = ref false ;;
let hyphenation_flag = ref true ;;



let main () =
  Arg.parse
    [ ("-f", Arg.String (fun s -> in_filename := s),
       "Specify an input file to check") ;
      ("-e", Arg.Unit (fun _ -> echo_on := true),
       "Force echo (word check is displayed)") ;
      ("-n", Arg.Unit (fun _ -> hyphenation_flag := false),
       "Disable hyphenation heuristic") ]
    (fun fname -> dic_name := fname)
    "Verify is words exist in a dictionary" ;
  let dic_file = open_in_bin !dic_name in
  let initial_base = ((input_value dic_file) : int)  in
  let dictionary = ((input_value dic_file) : Common.dictionary) in
  close_in dic_file ;
  let in_file = if !in_filename = "" then stdin else open_in !in_filename in
  let lexbuf = Lexing.from_channel in_file in
  try
    while true do
      begin
        match French.file lexbuf with
        | French.Word (word, _) ->
            if !echo_on then Printf.printf "'%s' : " word ;
            let word = String.uncapitalize word in
            if Exist.exists dictionary initial_base word
            then Printf.printf "Found\n"
            else
              begin
                if !hyphenation_flag then
                  (* Tests if the word is a combinaison of roots. *)
                  let sub_words = Hyphenation.split_string word in
                  if List.for_all (Exist.exists dictionary initial_base)
                      sub_words then
                    Printf.printf "Found AS COMBINAISON of several roots\n"
                  else Printf.printf "NOT found\n"
                else
                  Printf.printf "NOT found\n"
              end ;
            flush stdout
        | French.Forget garbage -> ()
      end
    done
  with End_of_file ->
    if !in_filename <> "" then close_in in_file ;
    Printf.printf"\n"
;;

try main () with Sys_error msg -> Printf.fprintf stderr "Error : %s\n" msg ;;
